// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SHARED_CLIENTID_H
#define SHARED_CLIENTID_H

#include <QString>
#include <QDataStream>

namespace Shared {

class ClientId {
public:
    ClientId();
    ClientId(const QString& node, const QString& verification, const QString& hash);
    ClientId(const ClientId& other);
    ClientId& operator = (const ClientId& other);

    bool operator == (const ClientId& other) const;
    bool operator != (const ClientId& other) const;
    bool operator < (const ClientId& other) const;
    bool operator > (const ClientId& other) const;
    bool operator <= (const ClientId& other) const;
    bool operator >= (const ClientId& other) const;

    bool valid() const;
    QString getId() const;

    QDataStream& operator << (QDataStream& stream);
    QDataStream& operator >> (QDataStream& stream) const;

public:
    QString node;
    QString verification;
    QString hash;
};

}

Q_DECLARE_METATYPE(Shared::ClientId)

QDataStream& operator << (QDataStream& stream, const Shared::ClientId& info);
QDataStream& operator >> (QDataStream& stream, Shared::ClientId& info);

#endif // SHARED_CLIENTID_H
